[^:]*: Assembler messages:
[^:]*:6: Error: unrecognised instruction size modifier .z
[^:]*:7: Error: junk found after instruction: mov.bc r1,r2
[^:]*:8: Error: junk found after instruction: mov.cd r1,r2
[^:]*:9: Error: junk found after instruction: mov.cd r1,r2
[^:]*:10: Warning: no size modifier after period, .w assumed
[^:]*:11: Error: instruction bis.a does not exist
[^:]*:16: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:16: Warning: a NOP might be needed before the EINT
[^:]*:25: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:25: Warning: a NOP might be needed before the EINT
[^:]*:29: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:31: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:32: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:33: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*:34: Warning: a NOP might be needed here because of successive changes in interrupt state
[^:]*: Warning: assembly finished without a possibly needed NOP instruction
