#source: pr13082-3.s
#name: PR ld/13082-3 (d)
#as: --64
#ld: -pie -melf_x86_64
#readelf: -r -x .data.rel.ro

There are no relocations in this file.

Hex dump of section '.data.rel.ro':
  0x[a-f0-9]+ 00000000 00000000                   ........
