.*: Assembler messages:
.*:17: Error: invalid unextended operand value
.*:19: Error: invalid unextended operand value
.*:21: Error: invalid unextended operand value
.*:23: Error: invalid unextended operand value
.*:25: Error: invalid unextended operand value
.*:27: Error: invalid unextended operand value
.*:29: Error: invalid unextended operand value
.*:31: Error: invalid unextended operand value
.*:33: Error: invalid unextended operand value
.*:35: Error: invalid unextended operand value
.*:37: Error: invalid unextended operand value
.*:39: Error: invalid unextended operand value
.*:41: Error: unsupported relocation
.*:43: Error: unsupported relocation
