#objdump: -d
#name:    
#source:  opr-idx3-xysp-24.s


dump.o:     file format elf32-s12z


Disassembly of section .text:

00000000 <.text>:
   0:	a6 c2 99 aa 	ld d6, \(-6706501,x\)
   4:	bb 
   5:	a7 d2 0a d3 	ld d7, \(709501,y\)
   9:	7d 
